@@ -26,9 +26,10 @@ wp_reset_vars(array('action', 'option_page'));
 
 $capability = 'manage_options';
 
-if ( empty($option_page) ) // This is for back compat and will eventually be removed.
+// This is for back compat and will eventually be removed.
+if ( empty($option_page) ) {
 	$option_page = 'options';
-else
+} else {
 
 	/**
 	 * Filter the capability required when using the Settings API.
@@ -41,9 +42,10 @@ else
 	 * @param string $capability The capability used for the page, which is manage_options by default.
 	 */
 	$capability = apply_filters( "option_page_capability_{$option_page}", $capability );
+}
 
 if ( !current_user_can( $capability ) )
-	wp_die(__('Cheatin&#8217; uh?'));
+	wp_die( __( 'Cheatin&#8217; uh?' ), 403 );
 
 // Handle admin email change requests
 if ( is_multisite() ) {
@@ -67,14 +69,14 @@ if ( is_multisite() ) {
 }
 
 if ( is_multisite() && !is_super_admin() && 'update' != $action )
-	wp_die(__('Cheatin&#8217; uh?'));
+	wp_die( __( 'Cheatin&#8217; uh?' ), 403 );
 
 $whitelist_options = array(
-	'general' => array( 'blogname', 'blogdescription', 'gmt_offset', 'date_format', 'time_format', 'start_of_week', 'timezone_string' ),
+	'general' => array( 'blogname', 'blogdescription', 'gmt_offset', 'date_format', 'time_format', 'start_of_week', 'timezone_string', 'WPLANG' ),
 	'discussion' => array( 'default_pingback_flag', 'default_ping_status', 'default_comment_status', 'comments_notify', 'moderation_notify', 'comment_moderation', 'require_name_email', 'comment_whitelist', 'comment_max_links', 'moderation_keys', 'blacklist_keys', 'show_avatars', 'avatar_rating', 'avatar_default', 'close_comments_for_old_posts', 'close_comments_days_old', 'thread_comments', 'thread_comments_depth', 'page_comments', 'comments_per_page', 'default_comments_page', 'comment_order', 'comment_registration' ),
 	'media' => array( 'thumbnail_size_w', 'thumbnail_size_h', 'thumbnail_crop', 'medium_size_w', 'medium_size_h', 'large_size_w', 'large_size_h', 'image_default_size', 'image_default_align', 'image_default_link_type' ),
 	'reading' => array( 'posts_per_page', 'posts_per_rss', 'rss_use_excerpt', 'show_on_front', 'page_on_front', 'page_for_posts', 'blog_public' ),
-	'writing' => array( 'use_smilies', 'default_category', 'default_email_category', 'use_balanceTags', 'default_link_category', 'default_post_format' )
+	'writing' => array( 'default_category', 'default_email_category', 'default_link_category', 'default_post_format' )
 );
 $whitelist_options['misc'] = $whitelist_options['options'] = $whitelist_options['privacy'] = array();
 
@@ -83,6 +85,11 @@ $mail_options = array('mailserver_url', 'mailserver_port', 'mailserver_login', '
 if ( ! in_array( get_option( 'blog_charset' ), array( 'utf8', 'utf-8', 'UTF8', 'UTF-8' ) ) )
 	$whitelist_options['reading'][] = 'blog_charset';
 
+if ( get_site_option( 'initial_db_version' ) < 32453 ) {
+	$whitelist_options['writing'][] = 'use_smilies';
+	$whitelist_options['writing'][] = 'use_balanceTags';
+}
+
 if ( !is_multisite() ) {
 	if ( !defined( 'WP_SITEURL' ) )
 		$whitelist_options['general'][] = 'siteurl';
@@ -105,14 +112,13 @@ if ( !is_multisite() ) {
 	}
 } else {
 	$whitelist_options['general'][] = 'new_admin_email';
-	$whitelist_options['general'][] = 'WPLANG';
 
 	/**
-	 * Toggle post-by-email functionality.
+	 * Filter whether the post-by-email functionality is enabled.
 	 *
 	 * @since 3.0.0
 	 *
-	 * @param bool True or false, based on whether post-by-email configuration is enabled or not.
+	 * @param bool $enabled Whether post-by-email configuration is enabled. Default true.
 	 */
 	if ( apply_filters( 'enable_post_by_email_configuration', true ) )
 		$whitelist_options['writing'] = array_merge($whitelist_options['writing'], $mail_options);
@@ -150,8 +156,8 @@ if ( 'update' == $action ) {
 		$options = $whitelist_options[ $option_page ];
 	}
 
-	// Handle custom date/time formats
 	if ( 'general' == $option_page ) {
+		// Handle custom date/time formats.
 		if ( !empty($_POST['date_format']) && isset($_POST['date_format_custom']) && '\c\u\s\t\o\m' == wp_unslash( $_POST['date_format'] ) )
 			$_POST['date_format'] = $_POST['date_format_custom'];
 		if ( !empty($_POST['time_format']) && isset($_POST['time_format_custom']) && '\c\u\s\t\o\m' == wp_unslash( $_POST['time_format'] ) )
@@ -162,12 +168,24 @@ if ( 'update' == $action ) {
 			$_POST['gmt_offset'] = preg_replace('/UTC\+?/', '', $_POST['gmt_offset']);
 			$_POST['timezone_string'] = '';
 		}
+
+		// Handle translation install.
+		if ( ! empty( $_POST['WPLANG'] ) && ( ! is_multisite() || is_super_admin() ) ) { // @todo: Skip if already installed
+			require_once( ABSPATH . 'wp-admin/includes/translation-install.php' );
+
+			if ( wp_can_install_language_pack() ) {
+				$language = wp_download_language_pack( $_POST['WPLANG'] );
+				if ( $language ) {
+					$_POST['WPLANG'] = $language;
+				}
+			}
+		}
 	}
 
 	if ( $options ) {
 		foreach ( $options as $option ) {
 			if ( $unregistered )
-				_deprecated_argument( 'options.php', '2.7', sprintf( __( 'The <code>%1$s</code> setting is unregistered. Unregistered settings are deprecated. See http://codex.wordpress.org/Settings_API' ), $option, $option_page ) );
+				_deprecated_argument( 'options.php', '2.7', sprintf( __( 'The <code>%1$s</code> setting is unregistered. Unregistered settings are deprecated. See https://codex.wordpress.org/Settings_API' ), $option, $option_page ) );
 
 			$option = trim( $option );
 			$value = null;
@@ -179,6 +197,14 @@ if ( 'update' == $action ) {
 			}
 			update_option( $option, $value );
 		}
+
+		// Switch translation in case WPLANG was changed.
+		$language = get_option( 'WPLANG' );
+		if ( $language ) {
+			load_default_textdomain( $language );
+		} else {
+			unload_textdomain( 'default' );
+		}
 	}
 
 	/**
@@ -200,12 +226,11 @@ if ( 'update' == $action ) {
 include( ABSPATH . 'wp-admin/admin-header.php' ); ?>
 
 <div class="wrap">
-<?php screen_icon(); ?>
-  <h2><?php esc_html_e('All Settings'); ?></h2>
+  <h1><?php esc_html_e( 'All Settings' ); ?></h1>
   <form name="form" action="options.php" method="post" id="all-options">
   <?php wp_nonce_field('options-options') ?>
   <input type="hidden" name="action" value="update" />
-  <input type='hidden' name='option_page' value='options' />
+  <input type="hidden" name="option_page" value="options" />
   <table class="form-table">
 <?php
 $options = $wpdb->get_results( "SELECT * FROM $wpdb->options ORDER BY option_name" );
@@ -216,7 +241,7 @@ foreach ( (array) $options as $option ) :
 		continue;
 	if ( is_serialized( $option->option_value ) ) {
 		if ( is_serialized_string( $option->option_value ) ) {
-			// this is a serialized string, so we should display it
+			// This is a serialized string, so we should display it.
 			$value = maybe_unserialize( $option->option_value );
 			$options_to_update[] = $option->option_name;
 			$class = 'all-options';
@@ -231,18 +256,19 @@ foreach ( (array) $options as $option ) :
 		$class = 'all-options';
 	}
 	$name = esc_attr( $option->option_name );
-	echo "
+	?>
 <tr>
-	<th scope='row'><label for='$name'>" . esc_html( $option->option_name ) . "</label></th>
-<td>";
-	if ( strpos( $value, "\n" ) !== false )
-		echo "<textarea class='$class' name='$name' id='$name' cols='30' rows='5'>" . esc_textarea( $value ) . "</textarea>";
-	else
-		echo "<input class='regular-text $class' type='text' name='$name' id='$name' value='" . esc_attr( $value ) . "'" . disabled( $disabled, true, false ) . " />";
-	echo "</td>
-</tr>";
-endforeach;
-?>
+	<th scope="row"><label for="<?php echo $name ?>"><?php echo esc_html( $option->option_name ); ?></label></th>
+<td>
+<?php if ( strpos( $value, "\n" ) !== false ) : ?>
+	<textarea class="<?php echo $class ?>" name="<?php echo $name ?>" id="<?php echo $name ?>" cols="30" rows="5"><?php
+		echo esc_textarea( $value );
+	?></textarea>
+	<?php else: ?>
+		<input class="regular-text <?php echo $class ?>" type="text" name="<?php echo $name ?>" id="<?php echo $name ?>" value="<?php echo esc_attr( $value ) ?>"<?php disabled( $disabled, true ) ?> />
+	<?php endif ?></td>
+</tr>
+<?php endforeach; ?>
   </table>
 
 <input type="hidden" name="page_options" value="<?php echo esc_attr( implode( ',', $options_to_update ) ); ?>" />
